@@ -16,7 +16,7 @@
 //! # Example
 //!
 //! ```ignore
-//! rules.extend(rewrite!("strength-reduction";  "(i32.shl ?x 1_i32)" <=> "(i32.mul ?x 2_i32)"));
+//! rules.extend(rewrite!("strength-reduction";  "(i32.shl ?x i32.const.1)" <=> "(i32.mul ?x i32.const.2)"));
 //! ```
 //!
 
@@ -30,7 +30,7 @@ use self::{
         analysis::PeepholeMutationAnalysis,
         encoder::{expr2wasm::ResourceRequest, Encoder},
         expr_enumerator::lazy_expand_aux,
-        lang::Lang,
+        lang::*,
     },
 };
 use super::{Mutator, OperatorAndByteOffset};
@@ -45,10 +45,12 @@ use wasm_encoder::{CodeSection, Function, GlobalSection, Instruction, Module, Va
 use wasmparser::{CodeSectionReader, FunctionBody, GlobalSectionReader, LocalsReader};
 
 /// This mutator applies a random peephole transformation to the input Wasm module
-#[derive(Clone, Copy)]
+#[derive(Clone)]
 pub struct PeepholeMutator {
     max_tree_depth: u32,
+    rules: Option<Vec<Rewrite<Lang, PeepholeMutationAnalysis>>>,
 }
+
 type EG = egg::EGraph<Lang, PeepholeMutationAnalysis>;
 
 impl PeepholeMutator {
@@ -56,6 +58,18 @@ impl PeepholeMutator {
     pub fn new(max_depth: u32) -> Self {
         PeepholeMutator {
             max_tree_depth: max_depth,
+            rules: None,
+        }
+    }
+
+    #[cfg(test)]
+    pub fn new_with_rules(
+        max_tree_depth: u32,
+        rules: Vec<Rewrite<Lang, PeepholeMutationAnalysis>>,
+    ) -> Self {
+        PeepholeMutator {
+            max_tree_depth,
+            rules: Some(rules),
         }
     }
 
@@ -106,12 +120,12 @@ impl PeepholeMutator {
     fn random_mutate<'a>(
         self,
         config: &'a mut WasmMutate,
-        rules: Vec<Rewrite<Lang, PeepholeMutationAnalysis>>,
+        rules: &[Rewrite<Lang, PeepholeMutationAnalysis>],
     ) -> Result<Box<dyn Iterator<Item = Result<Module>> + 'a>> {
         let code_section = config.info().get_code_section();
         let mut sectionreader = CodeSectionReader::new(code_section.data, 0)?;
         let function_count = sectionreader.get_count();
-        let mut function_to_mutate = config.rng().gen_range(0, function_count);
+        let mut function_to_mutate = config.rng().gen_range(0..function_count);
 
         let mut visited_functions = 0;
 
@@ -132,7 +146,7 @@ impl PeepholeMutator {
                 .collect::<wasmparser::Result<Vec<OperatorAndByteOffset>>>()?;
             let operatorscount = operators.len();
 
-            let mut opcode_to_mutate = config.rng().gen_range(0, operatorscount);
+            let mut opcode_to_mutate = config.rng().gen_range(0..operatorscount);
             log::trace!(
                 "Selecting operator {}/{} from function {}",
                 opcode_to_mutate,
@@ -204,16 +218,11 @@ impl PeepholeMutator {
                     function_to_mutate,
                 );
 
-                let analysis = PeepholeMutationAnalysis::new(
-                    config.info().global_types.clone(),
-                    locals.clone(),
-                    config.info().types_map.clone(),
-                    config.info().function_map.clone(),
-                );
+                let analysis = PeepholeMutationAnalysis::new(config.info(), locals.clone());
                 let runner = Runner::<Lang, PeepholeMutationAnalysis, ()>::new(analysis)
                     .with_iter_limit(1) // FIXME, the iterations should consume fuel from the actual mutator. Be careful with inner set time limits that can lead us to non-deterministic behavior
                     .with_expr(&start)
-                    .run(&rules);
+                    .run(rules);
                 let mut egraph = runner.egraph;
                 // In theory this will return the Id of the operator eterm
                 let root = egraph.add_expr(&start);
@@ -407,7 +416,7 @@ impl PeepholeMutator {
     fn mutate_with_rules<'a>(
         self,
         config: &'a mut WasmMutate,
-        rules: Vec<Rewrite<Lang, PeepholeMutationAnalysis>>,
+        rules: &[Rewrite<Lang, PeepholeMutationAnalysis>],
     ) -> Result<Box<dyn Iterator<Item = Result<Module>> + 'a>> {
         self.random_mutate(config, rules)
     }
@@ -419,12 +428,14 @@ impl Mutator for PeepholeMutator {
         self,
         config: &'a mut crate::WasmMutate,
     ) -> Result<Box<dyn Iterator<Item = Result<Module>> + 'a>> {
-        // Calculate here type related information for parameters, locals and returns
-        // This information could be passed to the conditions to check for type correctness rewriting
-        // Write the new rules in the rules.rs file
-        let rules = self.get_rules(config);
-
-        let modules = self.mutate_with_rules(config, rules)?;
+        let rules = match self.rules.clone() {
+            Some(rules) => rules,
+            // Calculate here type related information for parameters, locals and returns
+            // This information could be passed to the conditions to check for type correctness rewriting
+            // Write the new rules in the rules.rs file
+            None => self.get_rules(config),
+        };
+        let modules = self.mutate_with_rules(config, &rules)?;
 
         Ok(modules)
     }
@@ -616,8 +627,8 @@ mod tests {
                 (type (;0;) (func (result i32)))
                 (func (;0;) (type 0) (result i32)
                   (local i32 i32)
-                  i32.const 160268115
-                  i32.const -160268059
+                  i32.const -1985698784
+                  i32.const 1985698840
                   i32.add)
                 (export "exported_func" (func 0)))
             "#,
@@ -697,7 +708,7 @@ mod tests {
     #[test]
     fn test_peep_wrap() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("strength-undo";  "?x" => "(i32.add ?x 0_i32)" if is_type("?x", PrimitiveTypeInfo::I32)),
+            rewrite!("strength-undo";  "?x" => "(i32.add ?x i32.const.0)" if is_type("?x", PrimitiveTypeInfo::I32)),
         ];
 
         test_peephole_mutator(
@@ -717,22 +728,21 @@ mod tests {
         "#,
             rules,
             r#"
-            (module
-                (type (;0;) (func (result i32) ))
-                (func (;0;) (type 0) (result i32)
-                    (local i64)
-                    local.get 0
-                    i64.const 0
-                    i64.shl
-                    i32.wrap_i64
-                    i32.const -441701230
-                    i32.const 441701230
-                    i32.add
-                    i32.add
-                    i32.const 0
-                    i32.add)
-              (export "exported_func" (func 0)))
-            "#,
+        (module
+            (func (export "exported_func") (result i32) (local i64)
+                local.get 0
+                i64.const 0
+                i64.shl
+                i32.wrap_i64
+                i32.const -441701230
+                i32.const 0
+                i32.add
+                i32.const 441701230
+                i32.add
+                i32.add
+            )
+        )
+        "#,
             0,
         );
     }
@@ -740,7 +750,7 @@ mod tests {
     #[test]
     fn test_peep_irelop1() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] =
-            &[rewrite!("strength-undo";  "(i64.eqz ?x)" => "(i64.eq ?x 0_i64)")];
+            &[rewrite!("strength-undo";  "(i64.eqz ?x)" => "(i64.eq ?x i64.const.0)")];
 
         test_peephole_mutator(
             r#"
@@ -770,7 +780,7 @@ mod tests {
     #[test]
     fn test_peep_bug1() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("strength-undo";  "?x" => "(i32.shl ?x 0_i32)" if is_type("?x", PrimitiveTypeInfo::I32)),
+            rewrite!("strength-undo";  "?x" => "(i32.shl ?x i32.const.0)" if is_type("?x", PrimitiveTypeInfo::I32)),
         ];
 
         test_peephole_mutator(
@@ -927,7 +937,7 @@ mod tests {
         assert_eq!(can_mutate, true);
         let rules = mutator.get_rules(&wasmmutate);
 
-        for mutated in mutator.mutate_with_rules(&mut wasmmutate, rules).unwrap() {
+        for mutated in mutator.mutate_with_rules(&mut wasmmutate, &rules).unwrap() {
             let module = mutated.unwrap();
 
             let mut validator = wasmparser::Validator::new();
@@ -940,7 +950,7 @@ mod tests {
     #[test]
     fn test_mem_store1() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("rule";  "(i32.store.600.0.0 ?value ?offset)" => "(i32.store.0.0.0 ?value (i32.add ?offset 600_i32))" ),
+            rewrite!("rule";  "(i32.store.600.0.0 ?value ?offset)" => "(i32.store.0.0.0 ?value (i32.add ?offset i32.const.600))" ),
         ];
 
         test_peephole_mutator(
@@ -961,9 +971,9 @@ mod tests {
                 (type (;0;) (func (param i64 i32 f32)))
                 (func (;0;) (type 0) (param i64 i32 f32)
                   i32.const 100
+                  i32.const 200
                   i32.const 600
                   i32.add
-                  i32.const 200
                   i32.store align=1)
                 (memory (;0;) 0)
                 (export "\00" (memory 0)))
@@ -975,7 +985,7 @@ mod tests {
     #[test]
     fn test_peep_shl0() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("strength-undo3";  "(i64.shr_u ?x ?y)" => "(i64.shl (i64.shr_u ?x ?y) 0_i64)" ),
+            rewrite!("strength-undo3";  "(i64.shr_u ?x ?y)" => "(i64.shl (i64.shr_u ?x ?y) i64.const.0)" ),
         ];
 
         test_peephole_mutator(
@@ -1184,7 +1194,7 @@ mod tests {
     #[test]
     fn test_peep_idem3() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("idempotent-3";  "?x" => "(i32.add ?x 0_i32)" if is_type("?x", PrimitiveTypeInfo::I32)),
+            rewrite!("idempotent-3";  "?x" => "(i32.add ?x i32.const.0)" if is_type("?x", PrimitiveTypeInfo::I32)),
         ];
 
         test_peephole_mutator(
@@ -1213,8 +1223,8 @@ mod tests {
     #[test]
     fn test_peep_idem4() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("idempotent-4";  "?x" => "(i32.mul ?x 1_i32)" if is_type("?x", PrimitiveTypeInfo::I32)),
-            rewrite!("idempotent-4";  "?x" => "(i64.mul ?x 1_i32)" if is_type("?x", PrimitiveTypeInfo::I64)),
+            rewrite!("idempotent-4";  "?x" => "(i32.mul ?x i32.const.1)" if is_type("?x", PrimitiveTypeInfo::I32)),
+            rewrite!("idempotent-4";  "?x" => "(i64.mul ?x i32.const.1)" if is_type("?x", PrimitiveTypeInfo::I64)),
         ];
 
         test_peephole_mutator(
@@ -1302,7 +1312,7 @@ mod tests {
     #[test]
     fn test_peep_locals3() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] =
-            &[rewrite!("type1-1";  "(local.set.1 100_i32)" => "(local.set.1 0_i32)" )];
+            &[rewrite!("type1-1";  "(local.set.1 i32.const.100)" => "(local.set.1 i32.const.0)" )];
 
         test_peephole_mutator(
             r#"
@@ -1332,7 +1342,7 @@ mod tests {
     #[test]
     fn test_peep_functions() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] =
-            &[rewrite!("type1-1";  "(call.0 ?x ?y)" => "(call.0 1_i64 11_i32) " )];
+            &[rewrite!("type1-1";  "(call.0 ?x ?y)" => "(call.0 i64.const.1 i32.const.11) " )];
 
         test_peephole_mutator(
             r#"
@@ -1424,6 +1434,21 @@ mod tests {
                   local.get 6
                   i64.div_s
                   call 0
+                  local.get 6
+                  local.get 6
+                  i64.div_s
+                  local.get 6
+                  i64.div_s
+                  local.get 6
+                  i64.div_s
+                  local.get 6
+                  i64.div_s
+                  local.get 6
+                  i64.div_s
+                  local.get 6
+                  i64.div_s
+                  call 0
+                  i32.or
                   i32.or
                 )
             )
@@ -1465,7 +1490,7 @@ mod tests {
     #[test]
     fn test_peep_floats1() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] =
-            &[rewrite!("rule";  "1065353216_f32" => "0_f32" )];
+            &[rewrite!("rule";  "f32.const.1065353216" => "f32.const.0" )];
 
         test_peephole_mutator(
             r#"
@@ -1491,7 +1516,7 @@ mod tests {
     #[test]
     fn test_peep_globals1() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("mem-load-shift";  "?x" => "(i32.add ?x 0_i32)" if is_type("?x", PrimitiveTypeInfo::I32)),
+            rewrite!("mem-load-shift";  "?x" => "(i32.add ?x i32.const.0)" if is_type("?x", PrimitiveTypeInfo::I32)),
         ];
 
         test_peephole_mutator(
@@ -1523,7 +1548,7 @@ mod tests {
     #[test]
     fn test_peep_globals2() {
         let rules: &[Rewrite<super::Lang, PeepholeMutationAnalysis>] = &[
-            rewrite!("rule";  "?x" => "(i32.add ?x 0_i32)" if is_type("?x", PrimitiveTypeInfo::I32)),
+            rewrite!("rule";  "?x" => "(i32.add ?x i32.const.0)" if is_type("?x", PrimitiveTypeInfo::I32)),
         ];
 
         test_peephole_mutator(
@@ -1562,39 +1587,11 @@ mod tests {
         expected: &str,
         seed: u64,
     ) {
-        let original = &wat::parse_str(original).unwrap();
-        let info = ModuleInfo::new(original).unwrap();
+        let mut config = WasmMutate::default();
+        config.fuel(300);
+        config.seed(seed);
 
-        let mut wasmmutate = WasmMutate::default();
-        wasmmutate.fuel(300);
-        wasmmutate.info = Some(info);
-        let rnd = SmallRng::seed_from_u64(seed);
-        wasmmutate.rng = Some(rnd);
-
-        let mutator = PeepholeMutator::new(3);
-
-        let can_mutate = mutator.can_mutate(&wasmmutate);
-
-        assert_eq!(can_mutate, true);
-        let mut found = false;
-        for mutated in mutator
-            .mutate_with_rules(&mut wasmmutate, rules.to_vec())
-            .unwrap()
-        {
-            let mut validator = wasmparser::Validator::new();
-            let mutated_bytes = &mutated.unwrap().finish();
-            let text = wasmprinter::print_bytes(mutated_bytes).unwrap();
-            crate::validate(&mut validator, mutated_bytes);
-
-            let expected_bytes = &wat::parse_str(expected).unwrap();
-            let expectedtext = wasmprinter::print_bytes(expected_bytes).unwrap();
-
-            if expectedtext == text {
-                found = true;
-            }
-        }
-
-        // Assert that the passed mutation was found
-        assert!(found)
+        let mutator = PeepholeMutator::new_with_rules(3, rules.to_vec());
+        config.match_mutation(original, mutator, expected);
     }
 }
