@@ -5,7 +5,7 @@ use std::collections::HashMap;
 use egg::{Id, Language, RecExpr};
 use wasmparser::{Operator, Range};
 
-use crate::mutators::peephole::Lang;
+use crate::mutators::peephole::{Lang, MemArg, MemoryCopy, MemoryInit, TableCopy, TableInit};
 use crate::ModuleInfo;
 
 use crate::mutators::OperatorAndByteOffset;
@@ -191,22 +191,8 @@ impl MiniDFG {
     /// by cleaning spurious nodes
     pub fn get_expr(&self, at: usize) -> RecExpr<Lang> {
         let root = self.map[&at];
-        let enodes = self.entries[..=root]
-            .iter()
-            .map(|t| t.operator.clone())
-            .collect::<Vec<_>>();
-        let operands = self.entries[..=root]
-            .iter()
-            .map(|t| {
-                t.operator
-                    .children()
-                    .iter()
-                    .map(|i| Id::from(*i))
-                    .collect::<Vec<_>>()
-            })
-            .collect::<Vec<_>>();
-
-        build_expr(Id::from(root), &enodes, &operands)
+        let entries = &self.entries[..=root];
+        build_expr(Id::from(root), |id| &entries[usize::from(id)].operator)
     }
 }
 
@@ -399,7 +385,7 @@ impl<'a> DFGBuilder {
                             // Add this as a new operator
                             self.register_node(
                                 Lang::Call(
-                                    *function_index as usize,
+                                    *function_index,
                                     operands.iter().map(|i| Id::from(*i)).collect::<Vec<_>>(),
                                 ),
                                 idx,
@@ -442,173 +428,31 @@ impl<'a> DFGBuilder {
                     self.new_color();
                 }
 
-                Operator::I32Store { memarg } => {
-                    self.store(idx, |offset, value| Lang::I32Store {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::I64Store { memarg } => {
-                    self.store(idx, |offset, value| Lang::I64Store {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::F32Store { memarg } => {
-                    self.store(idx, |offset, value| Lang::F32Store {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::F64Store { memarg } => {
-                    self.store(idx, |offset, value| Lang::F64Store {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::I32Store8 { memarg } => {
-                    self.store(idx, |offset, value| Lang::I32Store8 {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::I32Store16 { memarg } => {
-                    self.store(idx, |offset, value| Lang::I32Store16 {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::I64Store8 { memarg } => {
-                    self.store(idx, |offset, value| Lang::I64Store8 {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::I64Store16 { memarg } => {
-                    self.store(idx, |offset, value| Lang::I64Store16 {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
-                Operator::I64Store32 { memarg } => {
-                    self.store(idx, |offset, value| Lang::I64Store32 {
-                        value_and_offset: [offset, value],
-                        static_offset: memarg.offset,
-                        align: memarg.align,
-                        mem: memarg.memory,
-                    });
-                    self.new_color();
-                }
+                Operator::I32Store { memarg } => self.store(idx, memarg, Lang::I32Store),
+                Operator::I64Store { memarg } => self.store(idx, memarg, Lang::I64Store),
+                Operator::F32Store { memarg } => self.store(idx, memarg, Lang::F32Store),
+                Operator::F64Store { memarg } => self.store(idx, memarg, Lang::F64Store),
+                Operator::I32Store8 { memarg } => self.store(idx, memarg, Lang::I32Store8),
+                Operator::I32Store16 { memarg } => self.store(idx, memarg, Lang::I32Store16),
+                Operator::I64Store8 { memarg } => self.store(idx, memarg, Lang::I64Store8),
+                Operator::I64Store16 { memarg } => self.store(idx, memarg, Lang::I64Store16),
+                Operator::I64Store32 { memarg } => self.store(idx, memarg, Lang::I64Store32),
 
                 // All memory loads
-                Operator::I32Load { memarg } => self.unop_cb(idx, |offset| Lang::I32Load {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load { memarg } => self.unop_cb(idx, |offset| Lang::I64Load {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::F32Load { memarg } => self.unop_cb(idx, |offset| Lang::F32Load {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::F64Load { memarg } => self.unop_cb(idx, |offset| Lang::F64Load {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I32Load8S { memarg } => self.unop_cb(idx, |offset| Lang::I32Load8S {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I32Load8U { memarg } => self.unop_cb(idx, |offset| Lang::I32Load8U {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I32Load16S { memarg } => self.unop_cb(idx, |offset| Lang::I32Load16S {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I32Load16U { memarg } => self.unop_cb(idx, |offset| Lang::I32Load16U {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load8S { memarg } => self.unop_cb(idx, |offset| Lang::I64Load8S {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load8U { memarg } => self.unop_cb(idx, |offset| Lang::I64Load8U {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load16S { memarg } => self.unop_cb(idx, |offset| Lang::I64Load16S {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load16U { memarg } => self.unop_cb(idx, |offset| Lang::I64Load16U {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load32S { memarg } => self.unop_cb(idx, |offset| Lang::I64Load32S {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
-                Operator::I64Load32U { memarg } => self.unop_cb(idx, |offset| Lang::I64Load32U {
-                    offset,
-                    static_offset: memarg.offset,
-                    align: memarg.align,
-                    mem: memarg.memory,
-                }),
+                Operator::I32Load { memarg } => self.load(idx, memarg, Lang::I32Load),
+                Operator::I64Load { memarg } => self.load(idx, memarg, Lang::I64Load),
+                Operator::F32Load { memarg } => self.load(idx, memarg, Lang::F32Load),
+                Operator::F64Load { memarg } => self.load(idx, memarg, Lang::F64Load),
+                Operator::I32Load8S { memarg } => self.load(idx, memarg, Lang::I32Load8S),
+                Operator::I32Load8U { memarg } => self.load(idx, memarg, Lang::I32Load8U),
+                Operator::I32Load16S { memarg } => self.load(idx, memarg, Lang::I32Load16S),
+                Operator::I32Load16U { memarg } => self.load(idx, memarg, Lang::I32Load16U),
+                Operator::I64Load8S { memarg } => self.load(idx, memarg, Lang::I64Load8S),
+                Operator::I64Load8U { memarg } => self.load(idx, memarg, Lang::I64Load8U),
+                Operator::I64Load16S { memarg } => self.load(idx, memarg, Lang::I64Load16S),
+                Operator::I64Load16U { memarg } => self.load(idx, memarg, Lang::I64Load16U),
+                Operator::I64Load32S { memarg } => self.load(idx, memarg, Lang::I64Load32S),
+                Operator::I64Load32U { memarg } => self.load(idx, memarg, Lang::I64Load32U),
 
                 Operator::I32Eqz => self.unop(idx, Lang::I32Eqz),
                 Operator::I64Eqz => self.unop(idx, Lang::I64Eqz),
@@ -777,26 +621,123 @@ impl<'a> DFGBuilder {
                         idx,
                     );
                 }
-                Operator::MemoryGrow { mem, mem_byte } => {
+                Operator::MemoryGrow { mem, mem_byte: _ } => {
                     let arg = self.pop_operand(idx, false);
+                    self.push_node(Lang::MemoryGrow(*mem, Id::from(arg)), idx);
+                }
+                Operator::MemorySize { mem, mem_byte: _ } => {
+                    self.push_node(Lang::MemorySize(*mem), idx);
+                }
+                Operator::TableGrow { table } => {
+                    let elem = self.pop_operand(idx, false);
+                    let size = self.pop_operand(idx, false);
                     self.push_node(
-                        Lang::MemoryGrow {
-                            mem: *mem,
-                            mem_byte: *mem_byte,
-                            by: Id::from(arg),
-                        },
+                        Lang::TableGrow(*table, [Id::from(size), Id::from(elem)]),
                         idx,
                     );
                 }
-                Operator::MemorySize { mem, mem_byte } => {
-                    self.push_node(
-                        Lang::MemorySize {
-                            mem: *mem,
-                            mem_byte: *mem_byte,
-                        },
+                Operator::TableSize { table } => {
+                    self.push_node(Lang::TableSize(*table), idx);
+                }
+
+                Operator::DataDrop { segment } => {
+                    self.empty_node(Lang::DataDrop(*segment), idx);
+                }
+
+                Operator::ElemDrop { segment } => {
+                    self.empty_node(Lang::ElemDrop(*segment), idx);
+                }
+
+                Operator::MemoryInit { mem, segment } => {
+                    let a = Id::from(self.pop_operand(idx, false));
+                    let b = Id::from(self.pop_operand(idx, false));
+                    let c = Id::from(self.pop_operand(idx, false));
+                    self.empty_node(
+                        Lang::MemoryInit(
+                            MemoryInit {
+                                memory: *mem,
+                                segment: *segment,
+                            },
+                            [c, b, a],
+                        ),
                         idx,
                     );
                 }
+                Operator::MemoryCopy { src, dst } => {
+                    let a = Id::from(self.pop_operand(idx, false));
+                    let b = Id::from(self.pop_operand(idx, false));
+                    let c = Id::from(self.pop_operand(idx, false));
+                    self.empty_node(
+                        Lang::MemoryCopy(
+                            MemoryCopy {
+                                src: *src,
+                                dst: *dst,
+                            },
+                            [c, b, a],
+                        ),
+                        idx,
+                    );
+                }
+
+                Operator::MemoryFill { mem } => {
+                    let a = Id::from(self.pop_operand(idx, false));
+                    let b = Id::from(self.pop_operand(idx, false));
+                    let c = Id::from(self.pop_operand(idx, false));
+                    self.empty_node(Lang::MemoryFill(*mem, [c, b, a]), idx);
+                }
+
+                Operator::TableInit { table, segment } => {
+                    let a = Id::from(self.pop_operand(idx, false));
+                    let b = Id::from(self.pop_operand(idx, false));
+                    let c = Id::from(self.pop_operand(idx, false));
+                    self.empty_node(
+                        Lang::TableInit(
+                            TableInit {
+                                table: *table,
+                                segment: *segment,
+                            },
+                            [c, b, a],
+                        ),
+                        idx,
+                    );
+                }
+                Operator::TableCopy {
+                    src_table,
+                    dst_table,
+                } => {
+                    let a = Id::from(self.pop_operand(idx, false));
+                    let b = Id::from(self.pop_operand(idx, false));
+                    let c = Id::from(self.pop_operand(idx, false));
+                    self.empty_node(
+                        Lang::TableCopy(
+                            TableCopy {
+                                src: *src_table,
+                                dst: *dst_table,
+                            },
+                            [c, b, a],
+                        ),
+                        idx,
+                    );
+                }
+
+                Operator::TableFill { table } => {
+                    let a = Id::from(self.pop_operand(idx, false));
+                    let b = Id::from(self.pop_operand(idx, false));
+                    let c = Id::from(self.pop_operand(idx, false));
+                    self.empty_node(Lang::TableFill(*table, [c, b, a]), idx);
+                }
+
+                Operator::TableGet { table } => {
+                    let arg = Id::from(self.pop_operand(idx, false));
+                    self.push_node(Lang::TableGet(*table, arg), idx);
+                }
+
+                Operator::TableSet { table } => {
+                    let arg1 = Id::from(self.pop_operand(idx, false));
+                    let arg2 = Id::from(self.pop_operand(idx, false));
+                    self.push_node(Lang::TableSet(*table, [arg2, arg1]), idx);
+                }
+
                 _ => {
                     // If the operator is not implemented, break the mutation of this Basic Block
                     return None;
@@ -815,6 +756,15 @@ impl<'a> DFGBuilder {
         self.unop_cb(idx, |id| op([id]))
     }
 
+    fn load(
+        &mut self,
+        idx: usize,
+        memarg: &wasmparser::MemoryImmediate,
+        op: fn(MemArg, Id) -> Lang,
+    ) {
+        self.unop_cb(idx, |id| op(memarg.into(), id))
+    }
+
     fn unop_cb(&mut self, idx: usize, op: impl Fn(Id) -> Lang) {
         let arg = self.pop_operand(idx, false);
         self.push_node(op(Id::from(arg)), idx);
@@ -830,14 +780,22 @@ impl<'a> DFGBuilder {
         self.push_node(op([Id::from(rightidx), Id::from(leftidx)]), idx);
     }
 
-    fn store(&mut self, idx: usize, op: impl Fn(Id, Id) -> Lang) {
+    fn store(
+        &mut self,
+        idx: usize,
+        memarg: &wasmparser::MemoryImmediate,
+        op: fn(MemArg, [Id; 2]) -> Lang,
+    ) {
         let leftidx = self.pop_operand(idx, false);
         let rightidx = self.pop_operand(idx, false);
 
         // The operands should not be the same
         assert_ne!(leftidx, rightidx);
 
-        self.empty_node(op(Id::from(rightidx), Id::from(leftidx)), idx);
+        self.empty_node(
+            op(memarg.into(), [Id::from(rightidx), Id::from(leftidx)]),
+            idx,
+        );
     }
 }
 
@@ -853,6 +811,16 @@ impl std::fmt::Display for MiniDFG {
     }
 }
 
+impl From<&wasmparser::MemoryImmediate> for MemArg {
+    fn from(mem: &wasmparser::MemoryImmediate) -> MemArg {
+        MemArg {
+            align: mem.align,
+            mem: mem.memory,
+            static_offset: mem.offset,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::DFGBuilder;
